/*
 * Copyright (c) 1997, 2014, Oracle and/or its affiliates. All rights reserved.
 * Copyright (c) 2015, 2016, Loongson Technology. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 *
 */

#include "precompiled.hpp"
#include "asm/macroAssembler.hpp"
#include "asm/macroAssembler.inline.hpp"
#include "memory/resourceArea.hpp"
#include "runtime/java.hpp"
#include "runtime/stubCodeGenerator.hpp"
#include "vm_version_mips.hpp"
#ifdef TARGET_OS_FAMILY_linux
# include "os_linux.inline.hpp"
#endif
#ifdef TARGET_OS_FAMILY_solaris
# include "os_solaris.inline.hpp"
#endif
#ifdef TARGET_OS_FAMILY_windows
# include "os_windows.inline.hpp"
#endif
#ifdef TARGET_OS_FAMILY_bsd
# include "os_bsd.inline.hpp"
#endif
/*
int VM_Version::_cpu;
int VM_Version::_model;
int VM_Version::_stepping;
int VM_Version::_cpuFeatures;
const char*           VM_Version::_features_str = "";
VM_Version::CpuidInfo VM_Version::_cpuid_info   = { 0, };

static BufferBlob* stub_blob;
static const int stub_size = 300;

extern "C" {
  typedef void (*getPsrInfo_stub_t)(void*);
}
static getPsrInfo_stub_t getPsrInfo_stub = NULL;
*/
int VM_Version::_features = VM_Version::unknown_m;
const char* VM_Version::_features_str = "";
/*
class VM_Version_StubGenerator: public StubCodeGenerator {
 public:

  VM_Version_StubGenerator(CodeBuffer *c) : StubCodeGenerator(c) {}

  address generate_getPsrInfo() {
  };
};


void VM_Version::get_processor_features() {
}
*/
void VM_Version::initialize() {

  _features = determine_features();
  //no need, Abstract_VM_Version already define it as false
  _supports_cx8 = true;

  //////////////////////add some other feature here//////////////////
#ifdef COMPILER2
  if (MaxVectorSize > 0) {
    if (!is_power_of_2(MaxVectorSize)) {
      warning("MaxVectorSize must be a power of 2");
      MaxVectorSize = 8;
    }
    if (MaxVectorSize > 0 && supports_ps()) {
      MaxVectorSize = 8;
    }
  }
/*
 *
 * 2017/5/17:
 * Vector optimization of MIPS works in most cases, but cannot pass hotspot/test/compiler/6340864/TestFloatVect.java.
 * Vector optimization was closed by default.
 * The reasons:
 * 1. The kernel does not have emulation of PS instructions yet, so the emulation of PS instructions must be done in JVM, see JVM_handle_linux_signal.
 * 2. It seems the gcc4.4.7 had some bug related to ucontext_t, which is used in signal handler to emulate PS instructions.
 */
  if (FLAG_IS_DEFAULT(MaxVectorSize)) {
    MaxVectorSize = 0;
  }

  if (is_gs464e()) {
    if (FLAG_IS_DEFAULT(UseCountTrailingZerosInstruction)) {
      FLAG_SET_DEFAULT(UseCountTrailingZerosInstruction, 1);
    }
  } else if (UseCountTrailingZerosInstruction) {
    if (!FLAG_IS_DEFAULT(UseCountTrailingZerosInstruction))
      warning("Only 3A2000/3000 CPUs support UseCountTrailingZerosInstruction");
    FLAG_SET_DEFAULT(UseCountTrailingZerosInstruction, 0);
  }
#endif
  UseSSE = 0; // Only on x86 and x64

  if (is_loongson()) {
    if (FLAG_IS_DEFAULT(UseLoongsonISA)) {
      FLAG_SET_DEFAULT(UseLoongsonISA, 1);
    }
  } else if (UseLoongsonISA) {
    if (!FLAG_IS_DEFAULT(UseLoongsonISA))
      warning("Only Loongson CPUs support LoongISA");
    FLAG_SET_DEFAULT(UseLoongsonISA, 0);
  }

  if (is_gs464e()) {
    if (FLAG_IS_DEFAULT(Use3A2000)) {
      FLAG_SET_DEFAULT(Use3A2000, 1);
    }
  } else if (Use3A2000) {
    if (!FLAG_IS_DEFAULT(Use3A2000))
      warning("Only 3A2000/3000 CPUs support this option");
    FLAG_SET_DEFAULT(Use3A2000, 0);
  }

  if (TieredCompilation) {
    if (!FLAG_IS_DEFAULT(TieredCompilation))
      warning("TieredCompilation not supported");
    FLAG_SET_DEFAULT(TieredCompilation, false);
  }

  if (UseG1GC) {
    if (!FLAG_IS_DEFAULT(UseG1GC))
      warning("UseG1GC not supported");
    FLAG_SET_DEFAULT(UseG1GC, false);
  }

  char buf[256];
  jio_snprintf(buf, sizeof(buf), "%s%s%s%s%s%s%s%s%s",
              (has_l2_cache() ? ", has_l2_cache" : ""),
              (has_16k_page() ? ", has_16k_page" : ""),
              (is_loongson()  ? ", on_loongson"  : ""),
              (is_gs464() ? ", 3A1000" : ""),
              (is_gs464v() ? ", 3B1500" : "" ),
              (is_gs464e() ? ", 3A2000/3A3000/3B2000/3B3000" : ""),
              (UseLoongsonISA ? ", UseLoongsonISA" : ""),
              (UseCountTrailingZerosInstruction ? ", UseCountTrailingZerosInstruction" : ""),
              (Use3A2000 ? ", Use3A2000" : ""));

  // buf is started with ", " or is empty
  _features_str = strdup(strlen(buf) > 2 ? buf + 2 : buf);

  if (FLAG_IS_DEFAULT(AllocatePrefetchStyle)) {
    FLAG_SET_DEFAULT(AllocatePrefetchStyle, 1);
  }

  if (FLAG_IS_DEFAULT(AllocatePrefetchLines)) {
    FLAG_SET_DEFAULT(AllocatePrefetchLines, 1);
  }

  if (FLAG_IS_DEFAULT(AllocatePrefetchStepSize)) {
    FLAG_SET_DEFAULT(AllocatePrefetchStepSize, 64);
  }

  if (FLAG_IS_DEFAULT(AllocatePrefetchDistance)) {
    FLAG_SET_DEFAULT(AllocatePrefetchDistance, 64);
  }

  if (FLAG_IS_DEFAULT(AllocateInstancePrefetchLines)) {
    FLAG_SET_DEFAULT(AllocateInstancePrefetchLines, 1);
  }

  NOT_PRODUCT( if (PrintMiscellaneous && Verbose) print_features(); );
}

void VM_Version::print_features() {
  tty->print_cr("Version:%s", cpu_features());
}

int VM_Version::determine_features() {
  //////////////////////add some other feature here//////////////////
  int features = platform_features(unknown_m);
  //spt_16k_page_m;
  return features;
}

static int saved_features = 0;

void VM_Version::allow_all() {
  saved_features = _features;
  _features      = all_features_m;
}

void VM_Version::revert() {
  _features = saved_features;
}
